@@ -6,18 +6,19 @@ import numpy as np
 from sklearn.utils import indices_to_mask
 from sklearn.linear_model import Lasso, LogisticRegression
 from sklearn.linear_model._coordinate_descent import _alpha_grid
-from sklearn.model_selection import KFold
+from sklearn.model_selection import KFold, cross_val_score
+from sklearn.metrics import accuracy_score
 
 from imodels.util.rule import Rule
 
 
 def score_precision_recall(X,
-              y,
-              rules: List[List[str]],
-              samples: List[List[int]],
-              features: List[List[int]],
-              feature_names: List[str],
-              oob: bool = True) -> List[Rule]:
+                           y,
+                           rules: List[List[str]],
+                           samples: List[List[int]],
+                           features: List[List[int]],
+                           feature_names: List[str],
+                           oob: bool = True) -> List[Rule]:
 
     scored_rules = []
 
@@ -67,59 +68,84 @@ def _eval_rule_perf(rule: str, X, y) -> Tuple[float, float]:
     return y_detected.mean(), float(true_pos) / pos
 
 
-def score_lasso(X, y, rules: List[str], alphas=None, cv=3,
-                prediction_task='regression',
-                max_rules=2000, random_state=None) -> Tuple[List[Rule], List[float], float]:
-    if alphas is None:
-        if prediction_task == 'regression':
-            alphas = _alpha_grid(X, y)
-        elif prediction_task == 'classification':
-            alphas = [1 / alpha
-                     for alpha in np.logspace(-4, 4, num=10, base=10)]
+def score_linear(X, y, rules: List[str], 
+                 penalty='l1',
+                 prediction_task='regression',
+                 max_rules=30,
+                 alpha=None,
+                 random_state=None) -> Tuple[List[Rule], List[float], float]:
 
-    coef_zero_threshold = 1e-6 / np.mean(np.abs(y))
-    mse_cv_scores = []
-    nonzero_rule_coefs_count = []
-    kf = KFold(cv)
-    
-    # alphas are sorted from most reg. to least reg.
-    for alpha in alphas: 
-        
-        if prediction_task == 'regression':
-            m = Lasso(alpha=alpha, random_state=random_state)
-        else:
-            m = LogisticRegression(penalty='l1', C=1/alpha, solver='liblinear')
-        mse_cv = 0
-        for train_index, test_index in kf.split(X):
-            X_train, X_test = X[train_index], X[test_index]
-            y_train, y_test = y[train_index], y[test_index]
-            m.fit(X_train, y_train)
-            mse_cv += np.mean((m.predict(X_test) - y_test) ** 2)
-        
-        m.fit(X, y)
-        
-        rule_count = np.sum(np.abs(m.coef_.flatten()) > coef_zero_threshold)
-        if rule_count > max_rules:
-            break
-        nonzero_rule_coefs_count.append(rule_count)
-        mse_cv_scores.append(mse_cv / cv)
-    
-    best_alpha = alphas[np.argmin(mse_cv_scores)]
-    if prediction_task == 'regression':
-        lscv = Lasso(alpha=best_alpha, random_state=random_state, max_iter=2000)
+    if alpha is not None and max_rules is None:
+        final_alpha = alpha
+    elif max_rules is not None and alpha is None:
+        final_alpha = get_best_alpha_under_max_rules(X, y, rules,
+                                                     penalty=penalty,
+                                                     prediction_task=prediction_task,
+                                                     max_rules=max_rules, 
+                                                     random_state=random_state)
     else:
-        lscv = LogisticRegression(penalty='l1', C=1/best_alpha, solver='liblinear',
+        raise ValueError("max_rules and alpha cannot be used together")
+
+
+    if prediction_task == 'regression':
+        lscv = Lasso(alpha=final_alpha, random_state=random_state, max_iter=2000)
+    else:
+        lscv = LogisticRegression(penalty=penalty, C=1/final_alpha, solver='liblinear',
                                   random_state=random_state, max_iter=200)
     lscv.fit(X, y)
 
     coef_ = lscv.coef_.flatten()
-    coefs = list(coef_[:len(coef_)-len(rules)])
+    coefs = list(coef_[:coef_.shape[0]-len(rules)])
     support = np.sum(X[:, -len(rules):], axis=0) / X.shape[0]
 
     nonzero_rules = []
+    coef_zero_threshold = 1e-6 / np.mean(np.abs(y))
     for r, w, s in zip(rules, coef_[-len(rules):], support):
         if abs(w) > coef_zero_threshold:
             nonzero_rules.append(Rule(r, args=[w], support=s))
             coefs.append(w)
     
     return nonzero_rules, coefs, lscv.intercept_
+
+
+def get_best_alpha_under_max_rules(X, y, rules: List[str],
+                                   penalty='l1',
+                                   prediction_task='regression',
+                                   max_rules=30,
+                                   random_state=None) -> float:
+    coef_zero_threshold = 1e-6 / np.mean(np.abs(y))
+    alpha_scores = []
+    nonzero_rule_coefs_count = []
+
+    if prediction_task == 'regression':
+        alphas = _alpha_grid(X, y)
+    elif prediction_task == 'classification':
+        alphas = [1 / alpha for alpha in np.logspace(-4, 4, num=10, base=10)]
+
+    # alphas are sorted from most reg. to least reg.
+    for alpha in alphas:
+
+        if prediction_task == 'regression':
+            m = Lasso(alpha=alpha, random_state=random_state)
+            fold_scores = cross_val_score(m, X, y, cv=4, scoring='neg_mean_squared_error')
+            alpha_scores.append(np.mean(fold_scores))
+        else:
+            m = LogisticRegression(penalty=penalty, C=1/alpha, solver='liblinear', random_state=random_state)
+            fold_scores = cross_val_score(m, X, y, cv=4, scoring='accuracy')
+            alpha_scores.append(np.mean(fold_scores))
+        
+        m.fit(X, y)
+        
+        rule_coefs = m.coef_.flatten()[:X.shape[1]-len(rules)]
+        rule_count = np.sum(np.abs(rule_coefs) > coef_zero_threshold)
+        if rule_count > max_rules:
+            break
+        nonzero_rule_coefs_count.append(rule_count)
+
+    # rare case in which diff alphas lead to identical scores
+    if np.all(alpha_scores == alpha_scores[0]):
+        best_alpha = alphas[len(alpha_scores) - 1]
+    else:
+        best_alpha = alphas[np.argmax(alpha_scores)]
+    
+    return best_alpha
